import { aws_ec2 as ec2, aws_iam as iam, aws_lambda as lambda, aws_logs as logs, RemovalPolicy } from 'aws-cdk-lib';
import { Construct } from 'constructs';
/**
 * Type of dependency packager. This affects timeouts and capabilities of the packager.
 */
export declare enum DependencyPackagerType {
    /**
     * Use Lambda function to package dependencies. It is much faster than the alternative, but limited to 15 minutes and can't build native extensions.
     */
    LAMBDA = 0,
    /**
     * Use CodeBuild to package dependencies. It is capable of everything your local machine can do, but takes a little longer to startup.
     */
    CODEBUILD = 1
}
export interface DependencyPackagerProps {
    /**
     * Type of dependency packager. Use Lambda for speed and CodeBuild for complex dependencies that require building native extensions.
     *
     * @default {@link DependencyPackagerType.LAMBDA}
     */
    readonly type?: DependencyPackagerType;
    /**
     * Target Lambda runtime. Packages will be installed for this runtime so make sure it fits your Lambda functions.
     */
    readonly runtime?: lambda.Runtime;
    /**
     * Target Lambda architecture. Packages will be installed for this architecture so make sure it fits your Lambda functions.
     */
    readonly architecture?: lambda.Architecture;
    /**
     * Additional commands to run before installing packages. Use this to authenticate your package repositories like CodeArtifact.
     *
     * @default []
     */
    readonly preinstallCommands?: string[];
    /**
     * VPC used for packager. Use this if your package repositories are only available from within a VPC.
     *
     * @default no VPC
     */
    readonly vpc?: ec2.IVpc;
    /**
     * VPC subnets used for packager.
     *
     * @default default subnets, if VPC is used
     */
    readonly subnetSelection?: ec2.SubnetSelection;
    /**
     * The number of days log events are kept in CloudWatch Logs. When updating
     * this property, unsetting it doesn't remove the log retention policy. To
     * remove the retention policy, set the value to `INFINITE`.
     *
     * @default logs.RetentionDays.ONE_MONTH
     */
    readonly logRetention?: logs.RetentionDays;
    /**
    * Removal policy for logs of image builds. If deployment fails on the custom resource, try setting this to `RemovalPolicy.RETAIN`. This way the CodeBuild logs can still be viewed, and you can see why the build failed.
    *
    * We try to not leave anything behind when removed. But sometimes a log staying behind is useful.
    *
    * @default RemovalPolicy.DESTROY
    */
    readonly logRemovalPolicy?: RemovalPolicy;
}
export interface LayerProps {
    /**
     * Always rebuild the layer, even when the dependencies definition files haven't changed.
     *
     * @default false
     */
    readonly alwaysRebuild?: boolean;
}
/**
 * @internal
 */
interface InternalBaseDependencyPackagerProps {
    readonly props?: DependencyPackagerProps;
    readonly runtimeFamily: lambda.RuntimeFamily;
    readonly defaultRuntime: lambda.Runtime;
    readonly codeBuildRuntimeInstallCommands: string[];
    readonly targetDirectory: string;
}
/**
 * @internal
 */
export declare class BaseDependencyPackager extends Construct implements iam.IGrantable, ec2.IConnectable {
    readonly internalProps: InternalBaseDependencyPackagerProps;
    readonly connections: ec2.Connections;
    readonly grantPrincipal: iam.IPrincipal;
    private readonly project?;
    private readonly packagesBucket;
    private readonly provider;
    private readonly targetDirectory;
    private readonly type;
    private readonly architecture;
    constructor(scope: Construct, id: string, internalProps: InternalBaseDependencyPackagerProps);
    /**
     * @internal
     */
    protected _newLayer(id: string, path: string, assetGenerator: (outputDir: string) => void, assetHash: string, commands: string[], layerProps?: LayerProps): lambda.LayerVersion;
    /**
     * @internal
     */
    protected _hashFiles(basePath: string, required: string[], optional?: string[]): string;
}
export {};
